@@ -19,7 +19,6 @@
 #include "gdk-pixdata.h"
 
 #include "gdk-pixbuf-private.h"
-#include "gdk-pixbuf-i18n.h"
 #include <string.h>
 
 #define APPEND g_string_append_printf
@@ -157,6 +156,13 @@ gdk_pixdata_serialize (const GdkPixdata *pixdata,
   return FALSE; \
 }
 
+static inline const guint8 *
+get_uint32 (const guint8 *stream, guint *result)
+{
+  *result = (stream[0] << 24) + (stream[1] << 16) + (stream[2] << 8) + stream[3];
+  return stream + 4;
+}
+
 /**
  * gdk_pixdata_deserialize:
  * @pixdata: a #GdkPixdata structure to be filled in.
@@ -181,7 +187,6 @@ gdk_pixdata_deserialize (GdkPixdata   *pixdata,
 			 const guint8 *stream,
 			 GError	     **error)
 {
-  guint32 *istream;
   guint color_type, sample_width, encoding;
 
   g_return_val_if_fail (pixdata != NULL, FALSE);
@@ -191,21 +196,14 @@ gdk_pixdata_deserialize (GdkPixdata   *pixdata,
 
 
   /* deserialize header */
-  istream = (guint32*) stream;
-  /*
-   * the deserialization of GdkPixdata will fail (at least on win32 with msvc 5.0) 
-   * with 'g_ntohl(*istream++)' because the guint32 istream pointer is only 
-   * incremented by 1 byte, if it is done within the g_ntohl() macro.
-   * Probably working around just another compiler bug ... --HB
-   */
-  pixdata->magic = g_ntohl (*istream); istream++;
-  pixdata->length = g_ntohl (*istream); istream++;
+  stream = get_uint32 (stream, &pixdata->magic);
+  stream = get_uint32 (stream, (guint32 *)&pixdata->length);
   if (pixdata->magic != GDK_PIXBUF_MAGIC_NUMBER || pixdata->length < GDK_PIXDATA_HEADER_LENGTH)
     return_header_corrupt (error);
-  pixdata->pixdata_type = g_ntohl (*istream); istream++;
-  pixdata->rowstride = g_ntohl (*istream); istream++;
-  pixdata->width = g_ntohl (*istream); istream++;
-  pixdata->height = g_ntohl (*istream);  istream++;
+  stream = get_uint32 (stream, &pixdata->pixdata_type);
+  stream = get_uint32 (stream, &pixdata->rowstride);
+  stream = get_uint32 (stream, &pixdata->width);
+  stream = get_uint32 (stream, &pixdata->height);
   if (pixdata->width < 1 || pixdata->height < 1 ||
       pixdata->rowstride < pixdata->width)
     return_header_corrupt (error);
@@ -222,7 +220,7 @@ gdk_pixdata_deserialize (GdkPixdata   *pixdata,
   /* deserialize pixel data */
   if (stream_length < pixdata->length - GDK_PIXDATA_HEADER_LENGTH)
     return_pixel_corrupt (error);
-  pixdata->pixel_data = (guint8*) istream;
+  pixdata->pixel_data = (guint8 *)stream;
 
   return TRUE;
 }
@@ -506,7 +504,7 @@ save_uchar (CSourceData *cdata,
 	  cdata->pad = FALSE;
 	}
     }
-  if (d < 33 || d > 126)
+  if (d < 33 || d > 126 || d == '?')
     {
       APPEND (gstring, "\\%o", d);
       cdata->pos += 1 + 1 + (d > 7) + (d > 63);
@@ -579,8 +577,8 @@ save_rle_decoder (GString     *gstring,
  * Generates C source code suitable for compiling images directly 
  * into programs. 
  *
- * Gtk+ ships with a program called gdk-pixbuf-csource which offers 
- * a cmdline interface to this functions.
+ * GTK+ ships with a program called <command>gdk-pixbuf-csource</command> 
+ * which offers a command line interface to this function.
  *
  * Returns: a newly-allocated string containing the C source form
  *   of @pixdata.
@@ -595,7 +593,7 @@ gdk_pixdata_to_csource (GdkPixdata        *pixdata,
   guint bpp, width, height, rowstride;
   gboolean rle_encoded;
   gchar *macro_name;
-  guint8 *img_buffer, *img_buffer_end, *stream;
+  guint8 *img_buffer, *img_buffer_end, *stream = NULL;
   guint stream_length;
   GString *gstring;
   
@@ -626,8 +624,7 @@ gdk_pixdata_to_csource (GdkPixdata        *pixdata,
   height = pixdata->height;
   rowstride = pixdata->rowstride;
   rle_encoded = (pixdata->pixdata_type & GDK_PIXDATA_ENCODING_RLE) > 0;
-  macro_name = g_strdup (name);
-  g_strup (macro_name);
+  macro_name = g_ascii_strup (name, -1);
 
   cdata.dump_macros = (dump_type & GDK_PIXDATA_DUMP_MACROS) > 0;
   cdata.dump_struct = (dump_type & GDK_PIXDATA_DUMP_PIXDATA_STRUCT) > 0;
@@ -638,7 +635,7 @@ gdk_pixdata_to_csource (GdkPixdata        *pixdata,
   cdata.dump_rle_decoder = (dump_type & GDK_PIXDATA_DUMP_RLE_DECODER) > 0;
   cdata.static_prefix = (dump_type & GDK_PIXDATA_DUMP_STATIC) ? "static " : "";
   cdata.const_prefix = (dump_type & GDK_PIXDATA_DUMP_CONST) ? "const " : "";
-  gstring = g_string_new ("");
+  gstring = g_string_new (NULL);
   cdata.gstring = gstring;
 
   if (!cdata.dump_macros && cdata.dump_gtypes)
@@ -709,9 +706,9 @@ gdk_pixdata_to_csource (GdkPixdata        *pixdata,
 	      cdata.static_prefix, cdata.const_prefix, name);
       APPEND (gstring, "  0x%x, /* Pixbuf magic: 'GdkP' */\n",
 	      GDK_PIXBUF_MAGIC_NUMBER);
-      APPEND (gstring, "  %u + %u, /* header length + pixel_data length */\n",
+      APPEND (gstring, "  %u + %lu, /* header length + pixel_data length */\n",
 	      GDK_PIXDATA_HEADER_LENGTH,
-	      rle_encoded ? img_buffer_end - img_buffer : rowstride * height);
+	      rle_encoded ? (glong)(img_buffer_end - img_buffer) : (glong)rowstride * height);
       APPEND (gstring, "  0x%x, /* pixdata_type */\n",
 	      pixdata->pixdata_type);
       APPEND (gstring, "  %u, /* rowstride */\n",
@@ -727,43 +724,55 @@ gdk_pixdata_to_csource (GdkPixdata        *pixdata,
       guint pix_length = img_buffer_end - img_buffer;
       
       stream = gdk_pixdata_serialize (pixdata, &stream_length);
+      img_buffer = stream;
       img_buffer_end = stream + stream_length;
 
+      APPEND (gstring, "#ifdef __SUNPRO_C\n");
+      APPEND (gstring, "#pragma align 4 (%s)\n", name);   
+      APPEND (gstring, "#endif\n");
+
+      APPEND (gstring, "#ifdef __GNUC__\n");
+      APPEND (gstring, "%s%s%s %s[] __attribute__ ((__aligned__ (4))) = \n",
+	      cdata.static_prefix, cdata.const_prefix,
+	      cdata.dump_gtypes ? "guint8" : "unsigned char",
+	      name);
+      APPEND (gstring, "#else\n");
       APPEND (gstring, "%s%s%s %s[] = \n",
 	      cdata.static_prefix, cdata.const_prefix,
 	      cdata.dump_gtypes ? "guint8" : "unsigned char",
 	      name);
-      APPEND (gstring, "( \"\"\n  /* Pixbuf magic (0x%x) */\n  \"",
+      APPEND (gstring, "#endif\n");
+
+      APPEND (gstring, "{ \"\"\n  /* Pixbuf magic (0x%x) */\n  \"",
 	      GDK_PIXBUF_MAGIC_NUMBER);
       cdata.pos = 3;
-      save_uchar (&cdata, *stream++); save_uchar (&cdata, *stream++);
-      save_uchar (&cdata, *stream++); save_uchar (&cdata, *stream++);
+      save_uchar (&cdata, *img_buffer++); save_uchar (&cdata, *img_buffer++);
+      save_uchar (&cdata, *img_buffer++); save_uchar (&cdata, *img_buffer++);
       APPEND (gstring, "\"\n  /* length: header (%u) + pixel_data (%u) */\n  \"",
 	      GDK_PIXDATA_HEADER_LENGTH,
 	      rle_encoded ? pix_length : rowstride * height);
       cdata.pos = 3;
-      save_uchar (&cdata, *stream++); save_uchar (&cdata, *stream++);
-      save_uchar (&cdata, *stream++); save_uchar (&cdata, *stream++);
+      save_uchar (&cdata, *img_buffer++); save_uchar (&cdata, *img_buffer++);
+      save_uchar (&cdata, *img_buffer++); save_uchar (&cdata, *img_buffer++);
       APPEND (gstring, "\"\n  /* pixdata_type (0x%x) */\n  \"",
 	      pixdata->pixdata_type);
       cdata.pos = 3;
-      save_uchar (&cdata, *stream++); save_uchar (&cdata, *stream++);
-      save_uchar (&cdata, *stream++); save_uchar (&cdata, *stream++);
+      save_uchar (&cdata, *img_buffer++); save_uchar (&cdata, *img_buffer++);
+      save_uchar (&cdata, *img_buffer++); save_uchar (&cdata, *img_buffer++);
       APPEND (gstring, "\"\n  /* rowstride (%u) */\n  \"",
 	      rowstride);
       cdata.pos = 3;
-      save_uchar (&cdata, *stream++); save_uchar (&cdata, *stream++);
-      save_uchar (&cdata, *stream++); save_uchar (&cdata, *stream++);
+      save_uchar (&cdata, *img_buffer++); save_uchar (&cdata, *img_buffer++);
+      save_uchar (&cdata, *img_buffer++); save_uchar (&cdata, *img_buffer++);
       APPEND (gstring, "\"\n  /* width (%u) */\n  \"", width);
       cdata.pos = 3;
-      save_uchar (&cdata, *stream++); save_uchar (&cdata, *stream++);
-      save_uchar (&cdata, *stream++); save_uchar (&cdata, *stream++);
+      save_uchar (&cdata, *img_buffer++); save_uchar (&cdata, *img_buffer++);
+      save_uchar (&cdata, *img_buffer++); save_uchar (&cdata, *img_buffer++);
       APPEND (gstring, "\"\n  /* height (%u) */\n  \"", height);
       cdata.pos = 3;
-      save_uchar (&cdata, *stream++); save_uchar (&cdata, *stream++);
-      save_uchar (&cdata, *stream++); save_uchar (&cdata, *stream++);
+      save_uchar (&cdata, *img_buffer++); save_uchar (&cdata, *img_buffer++);
+      save_uchar (&cdata, *img_buffer++); save_uchar (&cdata, *img_buffer++);
       APPEND (gstring, "\"\n  /* pixel_data: */\n");
-      img_buffer = stream;
     }
 
   /* pixel_data intro
@@ -801,7 +810,7 @@ gdk_pixdata_to_csource (GdkPixdata        *pixdata,
   if (cdata.dump_struct)
     APPEND (gstring, "\",\n};\n\n");
   if (cdata.dump_stream)
-    APPEND (gstring, "\");\n\n");
+    APPEND (gstring, "\"};\n\n");
 
   /* dump RLE decoder for macros
    */
@@ -814,45 +823,55 @@ gdk_pixdata_to_csource (GdkPixdata        *pixdata,
 
   /* cleanup
    */
+  g_free (stream);
   g_free (macro_name);
-  
+    
   return gstring;
 }
 
 /**
  * gdk_pixbuf_new_from_inline:
- * @data_length: Length in bytes of the @data argument
- * @data: Byte data containing a serialized GdkPixdata structure
+ * @data_length: Length in bytes of the @data argument or -1 to 
+ *    disable length checks
+ * @data: Byte data containing a serialized #GdkPixdata structure
  * @copy_pixels: Whether to copy the pixel data, or use direct pointers
  *               @data for the resulting pixbuf
- * @error: GError return location, may be %NULL to ignore errors
+ * @error: #GError return location, may be %NULL to ignore errors
  *
  * Create a #GdkPixbuf from a flat representation that is suitable for
  * storing as inline data in a program. This is useful if you want to
  * ship a program with images, but don't want to depend on any
  * external files.
  *
- * Gtk+ ships with a program called gdk-pixbuf-csource which allows
- * for conversion of #GdkPixbufs into such a inline representation.
- * In almost all cases, you should pass the --raw flag to
- * gdk-pixbuf-csource. A sample invocation would be:
+ * GTK+ ships with a program called <command>gdk-pixbuf-csource</command> 
+ * which allows for conversion of #GdkPixbufs into such a inline representation.
+ * In almost all cases, you should pass the <option>--raw</option> flag to
+ * <command>gdk-pixbuf-csource</command>. A sample invocation would be:
  *
+ * <informalexample><programlisting>
  *  gdk-pixbuf-csource --raw --name=myimage_inline myimage.png
+ * </programlisting></informalexample>
  * 
  * For the typical case where the inline pixbuf is read-only static data,
  * you don't need to copy the pixel data unless you intend to write to
- * it, so you can pass %FALSE for @copy_pixels.  (If you pass --rle to
- * gdk-pixbuf-csource, a copy will be made even if @copy_pixels is
- * %FALSE, so using this option is generally a bad idea.)
+ * it, so you can pass %FALSE for @copy_pixels.  (If you pass 
+ * <option>--rle</option> to <command>gdk-pixbuf-csource</command>, a copy 
+ * will be made even if @copy_pixels is %FALSE, so using this option is 
+ * generally a bad idea.)
  *
  * If you create a pixbuf from const inline data compiled into your
- * program, it's probably safe to ignore errors, since things will
- * always succeed.  For non-const inline data, you could get out of
- * memory. For untrusted inline data located at runtime, you could
- * have corrupt inline data in addition.
+ * program, it's probably safe to ignore errors and disable length checks, 
+ * since things will always succeed:
+ * <informalexample><programlisting>
+ * pixbuf = gdk_pixbuf_new_from_inline (-1, myimage_inline, NULL);
+ * </programlisting></informalexample>
+ *
+ * For non-const inline data, you could get out of memory. For untrusted 
+ * inline data located at runtime, you could have corrupt inline data in 
+ * addition.
  *
  * Return value: A newly-created #GdkPixbuf structure with a reference,
- *   count of 1, or %NULL if error is set.
+ *   count of 1, or %NULL if an error occurred.
  **/
 GdkPixbuf*
 gdk_pixbuf_new_from_inline (gint          data_length,
