@@ -123,8 +123,8 @@ gdk_selection_owner_set_for_display (GdkDisplay *display,
 
   GDK_NOTE (DND,
 	    (sel_name = gdk_atom_name (selection),
-	     g_print ("gdk_selection_owner_set: %#x %#x (%s)\n",
-		      (owner ? (guint) GDK_WINDOW_HWND (owner) : 0),
+	     g_print ("gdk_selection_owner_set: %p %#x (%s)\n",
+		      (owner ? GDK_WINDOW_HWND (owner) : NULL),
 		      (guint) selection, sel_name),
 	     g_free (sel_name)));
 
@@ -214,9 +214,9 @@ gdk_selection_owner_get_for_display (GdkDisplay *display,
 
   GDK_NOTE (DND,
 	    (sel_name = gdk_atom_name (selection),
-	     g_print ("gdk_selection_owner_get: %#x (%s) = %#x\n",
+	     g_print ("gdk_selection_owner_get: %#x (%s) = %p\n",
 		      (guint) selection, sel_name,
-		      (window ? (guint) GDK_WINDOW_HWND (window) : 0)),
+		      (window ? GDK_WINDOW_HWND (window) : NULL)),
 	     g_free (sel_name)));
 
   return window;
@@ -260,8 +260,8 @@ gdk_selection_convert (GdkWindow *requestor,
   GDK_NOTE (DND,
 	    (sel_name = gdk_atom_name (selection),
 	     tgt_name = gdk_atom_name (target),
-	     g_print ("gdk_selection_convert: %#x %#x (%s) %#x (%s)\n",
-		      (guint) GDK_WINDOW_HWND (requestor),
+	     g_print ("gdk_selection_convert: %p %#x (%s) %#x (%s)\n",
+		      GDK_WINDOW_HWND (requestor),
 		      (guint) selection, sel_name,
 		      (guint) target, tgt_name),
 	     g_free (sel_name),
@@ -465,8 +465,8 @@ gdk_selection_property_get (GdkWindow  *requestor,
   if (GDK_WINDOW_DESTROYED (requestor))
     return 0;
   
-  GDK_NOTE (DND, g_print ("gdk_selection_property_get: %#x\n",
-			   (guint) GDK_WINDOW_HWND (requestor)));
+  GDK_NOTE (DND, g_print ("gdk_selection_property_get: %p\n",
+			   GDK_WINDOW_HWND (requestor)));
 
   prop = g_hash_table_lookup (sel_prop_table, GDK_WINDOW_HWND (requestor));
 
@@ -494,8 +494,8 @@ _gdk_selection_property_delete (GdkWindow *window)
 {
   GdkSelProp *prop;
   
-  GDK_NOTE (DND, g_print ("_gdk_selection_property_delete: %#x\n",
-			   (guint) GDK_WINDOW_HWND (window)));
+  GDK_NOTE (DND, g_print ("_gdk_selection_property_delete: %p\n",
+			   GDK_WINDOW_HWND (window)));
 
   prop = g_hash_table_lookup (sel_prop_table, GDK_WINDOW_HWND (window));
   if (prop != NULL)
