@@ -48,17 +48,21 @@
 #include <errno.h>
 
 #include "gdk.h"		/* For gdk_error_trap_* / gdk_flush_* */
+#include "gdkx.h"
 #include "gdkimage.h"
 #include "gdkprivate.h"
 #include "gdkprivate-x11.h"
+#include "gdkdisplay-x11.h"
+#include "gdkscreen-x11.h"
 
 typedef struct _GdkImagePrivateX11     GdkImagePrivateX11;
 
 struct _GdkImagePrivateX11
 {
   XImage *ximage;
-  Display *xdisplay;
+  GdkScreen *screen;
   gpointer x_shm_info;
+  Pixmap shm_pixmap;
 };
 
 static GList *image_list = NULL;
@@ -138,27 +142,49 @@ _gdk_image_exit (void)
     }
 }
 
+/**
+ * gdk_image_new_bitmap:
+ * @visual: the #GdkVisual to use for the image.
+ * @data: the pixel data. 
+ * @width: the width of the image in pixels. 
+ * @height: the height of the image in pixels. 
+ * 
+ * Creates a new #GdkImage with a depth of 1 from the given data.
+ * <warning><para>THIS FUNCTION IS INCREDIBLY BROKEN. The passed-in data must 
+ * be allocated by malloc() (NOT g_malloc()) and will be freed when the 
+ * image is freed.</para></warning>
+ * 
+ * Return value: a new #GdkImage.
+ **/
 GdkImage *
-gdk_image_new_bitmap(GdkVisual *visual, gpointer data, gint w, gint h)
-/*
- * Desc: create a new bitmap image
- */
+gdk_image_new_bitmap(GdkVisual *visual, gpointer data, gint width, gint height)
 {
   Visual *xvisual;
   GdkImage *image;
+  GdkDisplay *display;
   GdkImagePrivateX11 *private;
+  
   image = g_object_new (gdk_image_get_type (), NULL);
   private = PRIVATE_DATA (image);
-  private->xdisplay = gdk_display;
+  private->screen = gdk_visual_get_screen (visual);
+  display = GDK_SCREEN_DISPLAY (private->screen);
+  
   image->type = GDK_IMAGE_NORMAL;
   image->visual = visual;
-  image->width = w;
-  image->height = h;
+  image->width = width;
+  image->height = height;
   image->depth = 1;
   image->bits_per_pixel = 1;
-  xvisual = ((GdkVisualPrivate*) visual)->xvisual;
-  private->ximage = XCreateImage(private->xdisplay, xvisual, 1, XYBitmap,
-				 0, 0, w ,h, 8, 0);
+  if (display->closed)
+    private->ximage = NULL;
+  else
+    {
+      xvisual = ((GdkVisualPrivate*) visual)->xvisual;
+      private->ximage = XCreateImage (GDK_SCREEN_XDISPLAY (private->screen),
+				      xvisual, 1, XYBitmap,
+				      0, 0, width, height, 8, 0);
+    }
+  
   private->ximage->data = data;
   private->ximage->bitmap_bit_order = MSBFirst;
   private->ximage->byte_order = MSBFirst;
@@ -169,62 +195,68 @@ gdk_image_new_bitmap(GdkVisual *visual, gpointer data, gint w, gint h)
   return(image);
 } /* gdk_image_new_bitmap() */
 
-static int
-gdk_image_check_xshm(Display *display)
-/* 
- * Desc: query the server for support for the MIT_SHM extension
- * Return:  0 = not available
- *          1 = shared XImage support available
- *          2 = shared Pixmap support available also
- */
-{
-#ifdef USE_SHM
-  int major, minor, ignore;
-  Bool pixmaps;
-  
-  if (XQueryExtension(display, "MIT-SHM", &ignore, &ignore, &ignore)) 
-    {
-      if (XShmQueryVersion(display, &major, &minor, &pixmaps )==True) 
-	{
-	  return (pixmaps==True) ? 2 : 1;
-	}
-    }
-#endif /* USE_SHM */
-  return 0;
-}
-
 void
-_gdk_windowing_image_init (void)
+_gdk_windowing_image_init (GdkDisplay *display)
 {
-  if (_gdk_use_xshm)
-    {
-      if (!gdk_image_check_xshm (gdk_display))
+  GdkDisplayX11 *display_x11 = GDK_DISPLAY_X11 (display);
+  
+    if (display_x11->use_xshm)
+      {
+#ifdef USE_SHM
+      Display *xdisplay = display_x11->xdisplay;
+      int major, minor, event_base;
+      Bool pixmaps;
+  
+      if (XShmQueryExtension (xdisplay) &&
+	  XShmQueryVersion (xdisplay, &major, &minor, &pixmaps))
 	{
-	  _gdk_use_xshm = False;
+	  display_x11->have_shm_pixmaps = pixmaps;
+	  event_base = XShmGetEventBase (xdisplay);
+
+	  _gdk_x11_register_event_type (display,
+					event_base, ShmNumberEvents);
 	}
-    }
+      else
+#endif /* USE_SHM */
+	display_x11->use_xshm = FALSE;
+      }
 }
 
 GdkImage*
-gdk_image_new (GdkImageType  type,
-	       GdkVisual    *visual,
-	       gint          width,
-	       gint          height)
+_gdk_image_new_for_depth (GdkScreen    *screen,
+			  GdkImageType  type,
+			  GdkVisual    *visual,
+			  gint          width,
+			  gint          height,
+			  gint          depth)
 {
   GdkImage *image;
   GdkImagePrivateX11 *private;
 #ifdef USE_SHM
   XShmSegmentInfo *x_shm_info;
 #endif /* USE_SHM */
-  Visual *xvisual;
+  Visual *xvisual = NULL;
+  GdkDisplayX11 *display_x11;
+  GdkScreenX11 *screen_x11;
 
+  g_return_val_if_fail (!visual || GDK_IS_VISUAL (visual), NULL);
+  g_return_val_if_fail (visual || depth != -1, NULL);
+  g_return_val_if_fail (GDK_IS_SCREEN (screen), NULL);
+  
+  screen_x11 = GDK_SCREEN_X11 (screen);
+  display_x11 = GDK_DISPLAY_X11 (screen_x11->display);
+  
+  if (visual)
+    depth = visual->depth;
+  
   switch (type)
     {
     case GDK_IMAGE_FASTEST:
-      image = gdk_image_new (GDK_IMAGE_SHARED, visual, width, height);
-
+      image = _gdk_image_new_for_depth (screen, GDK_IMAGE_SHARED, 
+					visual, width, height, depth);
       if (!image)
-	image = gdk_image_new (GDK_IMAGE_NORMAL, visual, width, height);
+	image = _gdk_image_new_for_depth (screen, GDK_IMAGE_NORMAL,
+					  visual, width, height, depth);
       break;
 
     default:
@@ -232,34 +264,35 @@ gdk_image_new (GdkImageType  type,
       
       private = PRIVATE_DATA (image);
 
-      private->xdisplay = gdk_display;
+      private->screen = screen;
 
       image->type = type;
       image->visual = visual;
       image->width = width;
       image->height = height;
-      image->depth = visual->depth;
+      image->depth = depth;
 
-      xvisual = ((GdkVisualPrivate*) visual)->xvisual;
+      if (visual)
+	xvisual = ((GdkVisualPrivate*) visual)->xvisual;
 
       switch (type)
 	{
 	case GDK_IMAGE_SHARED:
 #ifdef USE_SHM
-	  if (_gdk_use_xshm)
+	  if (display_x11->use_xshm)
 	    {
 	      private->x_shm_info = g_new (XShmSegmentInfo, 1);
 	      x_shm_info = private->x_shm_info;
 	      x_shm_info->shmid = -1;
 	      x_shm_info->shmaddr = (char*) -1;
 
-	      private->ximage = XShmCreateImage (private->xdisplay, xvisual, visual->depth,
+	      private->ximage = XShmCreateImage (screen_x11->xdisplay, xvisual, depth,
 						 ZPixmap, NULL, x_shm_info, width, height);
 	      if (private->ximage == NULL)
 		{
 		  g_warning ("XShmCreateImage failed");
-		  _gdk_use_xshm = FALSE;
-
+		  display_x11->use_xshm = FALSE;
+		  
 		  goto error;
 		}
 
@@ -277,7 +310,7 @@ gdk_image_new (GdkImageType  type,
 		  if (errno != EINVAL)
 		    {
 		      g_warning ("shmget failed: error %d (%s)", errno, g_strerror (errno));
-		      _gdk_use_xshm = FALSE;
+		      display_x11->use_xshm = FALSE;
 		    }
 
 		  goto error;
@@ -294,19 +327,19 @@ gdk_image_new (GdkImageType  type,
 		   * EMFILE, which would mean that we've exceeded the per-process
 		   * Shm segment limit.
 		   */
-		  _gdk_use_xshm = FALSE;
+		  display_x11->use_xshm = FALSE;
 		  goto error;
 		}
 
 	      gdk_error_trap_push ();
 
-	      XShmAttach (private->xdisplay, x_shm_info);
-	      XSync (private->xdisplay, False);
+	      XShmAttach (screen_x11->xdisplay, x_shm_info);
+	      XSync (screen_x11->xdisplay, False);
 
 	      if (gdk_error_trap_pop ())
 		{
 		  /* this is the common failure case so omit warning */
-		  _gdk_use_xshm = FALSE;
+		  display_x11->use_xshm = FALSE;
 		  goto error;
 		}
 	      
@@ -326,7 +359,7 @@ gdk_image_new (GdkImageType  type,
 	    goto error;
 	  break;
 	case GDK_IMAGE_NORMAL:
-	  private->ximage = XCreateImage (private->xdisplay, xvisual, visual->depth,
+	  private->ximage = XCreateImage (screen_x11->xdisplay, xvisual, depth,
 					  ZPixmap, 0, 0, width, height, 32, 0);
 
 	  /* Use malloc, not g_malloc here, because X will call free()
@@ -344,7 +377,7 @@ gdk_image_new (GdkImageType  type,
 
       if (image)
 	{
-	  image->byte_order = private->ximage->byte_order;
+	  image->byte_order = (private->ximage->byte_order == LSBFirst) ? GDK_LSB_FIRST : GDK_MSB_FIRST;
 	  image->mem = private->ximage->data;
 	  image->bpl = private->ximage->bytes_per_line;
 	  image->bpp = (private->ximage->bits_per_pixel + 7) / 8;
@@ -379,52 +412,165 @@ gdk_image_new (GdkImageType  type,
   return NULL;
 }
 
-GdkImage*
-_gdk_x11_get_image (GdkDrawable    *drawable,
-                    gint            x,
-                    gint            y,
-                    gint            width,
-                    gint            height)
+Pixmap
+_gdk_x11_image_get_shm_pixmap (GdkImage *image)
+{
+  GdkImagePrivateX11 *private = PRIVATE_DATA (image);
+  GdkDisplay *display = GDK_SCREEN_DISPLAY (private->screen);
+
+  if (display->closed)
+    return None;
+
+#ifdef USE_SHM  
+  /* Future: do we need one of these per-screen per-image? ShmPixmaps
+   * are the same for every screen, but can they be shared? Not a concern
+   * right now since we tie images to a particular screen.
+   */
+  if (!private->shm_pixmap && image->type == GDK_IMAGE_SHARED && 
+      GDK_DISPLAY_X11 (display)->have_shm_pixmaps)
+    private->shm_pixmap = XShmCreatePixmap (GDK_SCREEN_XDISPLAY (private->screen),
+					    GDK_SCREEN_XROOTWIN (private->screen),
+					    image->mem, private->x_shm_info, 
+					    image->width, image->height, image->depth);
+
+  return private->shm_pixmap;
+#else
+  return None;
+#endif    
+}
+
+static GdkImage*
+get_full_image (GdkDrawable    *drawable,
+		gint            src_x,
+		gint            src_y,
+		gint            width,
+		gint            height)
 {
   GdkImage *image;
   GdkImagePrivateX11 *private;
   GdkDrawableImplX11 *impl;
-  GdkVisual *visual;
-  gboolean have_grab;
-  GdkRectangle req;
-  GdkRectangle window_rect;
   XImage *ximage;
-  
-  g_return_val_if_fail (GDK_IS_DRAWABLE_IMPL_X11 (drawable), NULL);
-
-  visual = gdk_drawable_get_visual (drawable);
-
-  g_assert (visual || !GDK_IS_WINDOW_IMPL_X11 (drawable));
 
   impl = GDK_DRAWABLE_IMPL_X11 (drawable);
   
+  ximage = XGetImage (GDK_SCREEN_XDISPLAY (impl->screen),
+		      impl->xid,
+		      src_x, src_y, width, height,
+		      AllPlanes, ZPixmap);
+  
+  if (!ximage)
+    return NULL;
+  
+  image = g_object_new (gdk_image_get_type (), NULL);
+  
+  private = PRIVATE_DATA (image);
+  
+  private->screen = impl->screen;
+  private->ximage = ximage;
+  
+  image->type = GDK_IMAGE_NORMAL;
+  image->visual = gdk_drawable_get_visual (drawable); /* could be NULL */
+  image->width = width;
+  image->height = height;
+  image->depth = gdk_drawable_get_depth (drawable);
+  
+  image->mem = private->ximage->data;
+  image->bpl = private->ximage->bytes_per_line;
+  image->bits_per_pixel = private->ximage->bits_per_pixel;
+  image->bpp = (private->ximage->bits_per_pixel + 7) / 8;
+  image->byte_order = (private->ximage->byte_order == LSBFirst) ? GDK_LSB_FIRST : GDK_MSB_FIRST;
+  
+  return image;
+}
+
+GdkImage*
+_gdk_x11_copy_to_image (GdkDrawable    *drawable,
+			GdkImage       *image,
+			gint            src_x,
+			gint            src_y,
+			gint            dest_x,
+			gint            dest_y,
+			gint            width,
+			gint            height)
+{
+  GdkImagePrivateX11 *private;
+  GdkDrawableImplX11 *impl;
+  GdkVisual *visual;
+  GdkDisplay *display;
+  Display *xdisplay;
+  gboolean have_grab;
+  GdkRectangle req;
+  GdkRectangle window_rect;
+  Pixmap shm_pixmap = None;
+  gboolean success = TRUE;
+  
+  g_return_val_if_fail (GDK_IS_DRAWABLE_IMPL_X11 (drawable), NULL);
+  g_return_val_if_fail (image != NULL || (dest_x == 0 && dest_y == 0), NULL);
+
+  visual = gdk_drawable_get_visual (drawable);
+  impl = GDK_DRAWABLE_IMPL_X11 (drawable);
+  display = gdk_drawable_get_display (drawable);
+  xdisplay = gdk_x11_display_get_xdisplay (display);
+
+  if (display->closed)
+    return NULL;
+  
   have_grab = FALSE;
 
+#define UNGRAB() G_STMT_START {					\
+    if (have_grab) {						\
+      gdk_x11_display_ungrab (display);				\
+      have_grab = FALSE; }					\
+  } G_STMT_END
+
+  if (!image && !GDK_IS_WINDOW_IMPL_X11 (drawable))
+    return get_full_image (drawable, src_x, src_y, width, height);
+
+  if (image && image->type == GDK_IMAGE_SHARED)
+    {
+      shm_pixmap = _gdk_x11_image_get_shm_pixmap (image);
+      if (shm_pixmap)
+	{
+	  GC xgc;
+	  XGCValues values;
+
+	  /* Again easy, we can just XCopyArea, and don't have to worry about clipping
+	   */
+	  values.subwindow_mode = IncludeInferiors;
+	  xgc = XCreateGC (xdisplay, impl->xid, GCSubwindowMode, &values);
+	  
+	  XCopyArea (xdisplay, impl->xid, shm_pixmap, xgc,
+		     src_x, src_y, width, height, dest_x, dest_y);
+	  XSync (xdisplay, FALSE);
+	  
+	  XFreeGC (xdisplay, xgc);
+	  
+	  return image;
+	}
+    }
+
+  /* Now the general case - we may have to worry about clipping to the screen
+   * bounds, in which case we'll have to grab the server and only get a piece
+   * of the window.
+   */
   if (GDK_IS_WINDOW_IMPL_X11 (drawable))
     {
       GdkRectangle screen_rect;
       Window child;
 
-      g_assert (visual);
-
       have_grab = TRUE;
-      gdk_x11_grab_server ();
+      gdk_x11_display_grab (display);
 
       /* Translate screen area into window coordinates */
-      XTranslateCoordinates (gdk_display,
-			     _gdk_root_window,
-                             impl->xid,
+      XTranslateCoordinates (xdisplay,
+			     GDK_SCREEN_XROOTWIN (impl->screen),
+			     impl->xid,
 			     0, 0, 
 			     &screen_rect.x, &screen_rect.y, 
 			     &child);
 
-      screen_rect.width = gdk_screen_width ();
-      screen_rect.height = gdk_screen_height ();
+      screen_rect.width = gdk_screen_get_width (impl->screen);
+      screen_rect.height = gdk_screen_get_height (impl->screen);
       
       gdk_error_trap_push ();
 
@@ -443,13 +589,7 @@ _gdk_x11_get_image (GdkDrawable    *drawable,
       if (gdk_error_trap_pop () ||
           !gdk_rectangle_intersect (&window_rect, &screen_rect, 
                                     &window_rect))
-        {
-          if (have_grab)
-            gdk_x11_ungrab_server ();
-          return image = gdk_image_new (GDK_IMAGE_FASTEST,
-                                        visual,
-                                        width, height);
-        }
+	goto out;
     }
   else
     {
@@ -460,8 +600,8 @@ _gdk_x11_get_image (GdkDrawable    *drawable,
 			     &window_rect.height);
     }
       
-  req.x = x;
-  req.y = y;
+  req.x = src_x;
+  req.y = src_y;
   req.width = width;
   req.height = height;
   
@@ -470,97 +610,66 @@ _gdk_x11_get_image (GdkDrawable    *drawable,
    * For pixmaps this is all of the pixmap, for windows it is just 
    * the onscreen part.
    */
-  if (!gdk_rectangle_intersect (&req, &window_rect, &req) && visual) 
-    {      
-      if (have_grab)
-	gdk_x11_ungrab_server ();
-      return image = gdk_image_new (GDK_IMAGE_FASTEST,
-                                    visual,
-                                    width, height);
-    }
+  if (!gdk_rectangle_intersect (&req, &window_rect, &req))
+    goto out;
 
-  if (req.x != x || req.y != y)
+  gdk_error_trap_push ();
+  
+  if (!image &&
+      req.x == src_x && req.y == src_y && req.width == width && req.height == height)
     {
-      g_assert (GDK_IS_WINDOW (drawable));
-      g_assert (visual);
-
-      image = gdk_image_new (GDK_IMAGE_FASTEST,
-                             visual,
-                             width, height);
-      if (image == NULL)
-        {
-          if (have_grab)
-            gdk_x11_ungrab_server ();
-          return NULL;
-        }
-
-      private = PRIVATE_DATA (image);
-
-      gdk_error_trap_push ();
-
-      ximage = XGetSubImage (impl->xdisplay,
-                             impl->xid,
-                             req.x, req.y, req.width, req.height,
-                             AllPlanes, ZPixmap,
-                             private->ximage,
-                             req.x - x, req.y - y);
-
-      if (have_grab)
-        {
-          gdk_x11_ungrab_server ();
-          have_grab = FALSE;
-        }
-      
-      if (gdk_error_trap_pop () || ximage == NULL)
-        {
-          g_object_unref (G_OBJECT (image));
-          return NULL;
-        }
-
-      g_assert (ximage == private->ximage);
+      image = get_full_image (drawable, src_x, src_y, width, height);
+      if (!image)
+	success = FALSE;
     }
   else
     {
-      /* Here we ignore the req.width, req.height -
-       * XGetImage() will do the right thing without
-       * them.
-       */
-      ximage = XGetImage (impl->xdisplay,
-                          impl->xid,
-                          x, y, width, height,
-                          AllPlanes, ZPixmap);
-
-      if (have_grab)
-        {
-          gdk_x11_ungrab_server ();
-          have_grab = FALSE;
-        }      
-
-      if (!ximage)
-        return NULL;
-
-      image = g_object_new (gdk_image_get_type (), NULL);
+      gboolean created_image = FALSE;
+      
+      if (!image)
+	{
+	  image = _gdk_image_new_for_depth (impl->screen, GDK_IMAGE_NORMAL, 
+					    visual, width, height,
+					    gdk_drawable_get_depth (drawable));
+	  created_image = TRUE;
+	}
 
       private = PRIVATE_DATA (image);
 
-      private->xdisplay = gdk_display;
-      private->ximage = ximage;
-
-      image->type = GDK_IMAGE_NORMAL;
-      image->visual = visual; /* May be NULL */
-      image->width = width;
-      image->height = height;
-      image->depth = gdk_drawable_get_depth (drawable);
-
-      image->mem = private->ximage->data;
-      image->bpl = private->ximage->bytes_per_line;
-      image->bits_per_pixel = private->ximage->bits_per_pixel;
-      image->bpp = (private->ximage->bits_per_pixel + 7) / 8;
-      image->byte_order = private->ximage->byte_order; 
+      /* In the ShmImage but no ShmPixmap case, we could use XShmGetImage when
+       * we are getting the entire image.
+       */
+      if (XGetSubImage (xdisplay, impl->xid,
+			req.x, req.y, req.width, req.height,
+			AllPlanes, ZPixmap,
+			private->ximage,
+			dest_x + req.x - src_x, dest_y + req.y - src_y) == None)
+	{
+	  if (created_image)
+	    g_object_unref (image);
+	  image = NULL;
+	  success = FALSE;
+	}
     }
 
-  g_assert (!have_grab);
+ out:
   
+  if (have_grab)
+    {				
+      gdk_x11_display_ungrab (display);
+      have_grab = FALSE;
+    }
+  
+  gdk_error_trap_pop ();
+
+  if (success && !image)
+    {
+      /* We "succeeded", but could get no content for the image so return junk */
+      image = _gdk_image_new_for_depth (impl->screen, GDK_IMAGE_NORMAL, 
+					visual, width, height,
+					gdk_drawable_get_depth (drawable));
+    }
+      
   return image;
 }
 
@@ -573,10 +682,15 @@ gdk_image_get_pixel (GdkImage *image,
   GdkImagePrivateX11 *private;
 
   g_return_val_if_fail (GDK_IS_IMAGE (image), 0);
+  g_return_val_if_fail (x >= 0 && x < image->width, 0);
+  g_return_val_if_fail (y >= 0 && y < image->height, 0);
 
   private = PRIVATE_DATA (image);
 
-  pixel = XGetPixel (private->ximage, x, y);
+  if (!private->screen->closed)
+    pixel = XGetPixel (private->ximage, x, y);
+  else
+    pixel = 0;
 
   return pixel;
 }
@@ -590,10 +704,13 @@ gdk_image_put_pixel (GdkImage *image,
   GdkImagePrivateX11 *private;
 
   g_return_if_fail (GDK_IS_IMAGE (image));
+  g_return_if_fail (x >= 0 && x < image->width);
+  g_return_if_fail (y >= 0 && y < image->height);
 
   private = PRIVATE_DATA (image);
 
-  pixel = XPutPixel (private->ximage, x, y, pixel);
+  if (!private->screen->closed)
+    pixel = XPutPixel (private->ximage, x, y, pixel);
 }
 
 static void
@@ -619,23 +736,31 @@ gdk_x11_image_destroy (GdkImage *image)
       switch (image->type)
 	{
 	case GDK_IMAGE_NORMAL:
-	  XDestroyImage (private->ximage);
+	  if (!private->screen->closed)
+	    XDestroyImage (private->ximage);
 	  break;
 	  
 	case GDK_IMAGE_SHARED:
 #ifdef USE_SHM
-	  gdk_flush();
-	  
-	  XShmDetach (private->xdisplay, private->x_shm_info);
-	  XDestroyImage (private->ximage);
+	  if (!private->screen->closed)
+	    {
+	      gdk_display_sync (GDK_SCREEN_DISPLAY (private->screen));
+
+	      if (private->shm_pixmap)
+		XFreePixmap (GDK_SCREEN_XDISPLAY (private->screen), private->shm_pixmap);
+	  	  
+	      XShmDetach (GDK_SCREEN_XDISPLAY (private->screen), private->x_shm_info);
+	      XDestroyImage (private->ximage);
+	    }
 	  
+	  image_list = g_list_remove (image_list, image);
+
 	  x_shm_info = private->x_shm_info;
 	  shmdt (x_shm_info->shmaddr);
 	  
 	  g_free (private->x_shm_info);
 	  private->x_shm_info = NULL;
-	  
-	  image_list = g_list_remove (image_list, image);
+
 #else /* USE_SHM */
 	  g_error ("trying to destroy shared memory image when gdk was compiled without shared memory support");
 #endif /* USE_SHM */
@@ -650,6 +775,14 @@ gdk_x11_image_destroy (GdkImage *image)
   image->windowing_data = NULL;
 }
 
+/**
+ * gdk_x11_image_get_xdisplay:
+ * @image: a #GdkImage.
+ * 
+ * Returns the display of a #GdkImage.
+ * 
+ * Return value: an Xlib <type>Display*</type>.
+ **/
 Display *
 gdk_x11_image_get_xdisplay (GdkImage *image)
 {
@@ -659,9 +792,17 @@ gdk_x11_image_get_xdisplay (GdkImage *image)
 
   private = PRIVATE_DATA (image);
 
-  return private->xdisplay;
+  return GDK_SCREEN_XDISPLAY (private->screen);
 }
 
+/**
+ * gdk_x11_image_get_ximage:
+ * @image: a #GdkImage.
+ * 
+ * Returns the X image belonging to a #GdkImage.
+ * 
+ * Return value: an <type>XImage*</type>.
+ **/
 XImage *
 gdk_x11_image_get_ximage (GdkImage *image)
 {
@@ -671,5 +812,30 @@ gdk_x11_image_get_ximage (GdkImage *image)
 
   private = PRIVATE_DATA (image);
 
-  return private->ximage;
+  if (private->screen->closed)
+    return NULL;
+  else
+    return private->ximage;
 }
+
+gint
+_gdk_windowing_get_bits_for_depth (GdkDisplay *display,
+				   gint        depth)
+{
+  XPixmapFormatValues *formats;
+  gint count, i;
+
+  formats = XListPixmapFormats (GDK_DISPLAY_XDISPLAY (display), &count);
+  
+  for (i = 0; i < count; i++)
+    if (formats[i].depth == depth)
+      {
+	gint result = formats[i].bits_per_pixel;
+	XFree (formats);
+	return result;
+      }
+
+  g_assert_not_reached ();
+  return -1;
+}
+
