@@ -19,6 +19,7 @@
 
 #include "gtkcellrenderer.h"
 #include "gtkintl.h"
+#include "gtktreeprivate.h"
 
 static void gtk_cell_renderer_init       (GtkCellRenderer      *cell);
 static void gtk_cell_renderer_class_init (GtkCellRendererClass *class);
@@ -30,6 +31,8 @@ static void gtk_cell_renderer_set_property  (GObject              *object,
 					     guint                 param_id,
 					     const GValue         *value,
 					     GParamSpec           *pspec);
+static void set_cell_bg_color               (GtkCellRenderer      *cell,
+					     GdkColor             *color);
 
 
 enum {
@@ -43,14 +46,16 @@ enum {
   PROP_WIDTH,
   PROP_HEIGHT,
   PROP_IS_EXPANDER,
-  PROP_IS_EXPANDED
+  PROP_IS_EXPANDED,
+  PROP_CELL_BACKGROUND,
+  PROP_CELL_BACKGROUND_GDK,
+  PROP_CELL_BACKGROUND_SET
 };
 
-
-GtkType
+GType
 gtk_cell_renderer_get_type (void)
 {
-  static GtkType cell_type = 0;
+  static GType cell_type = 0;
 
   if (!cell_type)
     {
@@ -63,11 +68,13 @@ gtk_cell_renderer_get_type (void)
 	NULL,		/* class_finalize */
 	NULL,		/* class_data */
         sizeof (GtkCellRenderer),
-	0,
+	0,		/* n_preallocs */
         (GInstanceInitFunc) gtk_cell_renderer_init,
+	NULL,		/* value_table */
       };
 
-      cell_type = g_type_register_static (GTK_TYPE_OBJECT, "GtkCellRenderer", &cell_info, 0);
+      cell_type = g_type_register_static (GTK_TYPE_OBJECT, "GtkCellRenderer", 
+					  &cell_info, G_TYPE_FLAG_ABSTRACT);
     }
 
   return cell_type;
@@ -76,6 +83,8 @@ gtk_cell_renderer_get_type (void)
 static void
 gtk_cell_renderer_init (GtkCellRenderer *cell)
 {
+  GtkCellRendererInfo *cellinfo;
+
   cell->mode = GTK_CELL_RENDERER_MODE_INERT;
   cell->visible = TRUE;
   cell->width = -1;
@@ -84,6 +93,10 @@ gtk_cell_renderer_init (GtkCellRenderer *cell)
   cell->yalign = 0.5;
   cell->xpad = 0;
   cell->ypad = 0;
+
+  cellinfo = g_new0 (GtkCellRendererInfo, 1);
+  g_object_set_data_full (G_OBJECT (cell),
+		          GTK_CELL_RENDERER_INFO_KEY, cellinfo, g_free);
 }
 
 static void
@@ -120,7 +133,7 @@ gtk_cell_renderer_class_init (GtkCellRendererClass *class)
 				   PROP_XALIGN,
 				   g_param_spec_float ("xalign",
 						       _("xalign"),
-						       _("The x-align."),
+						       _("The x-align"),
 						       0.0,
 						       1.0,
 						       0.0,
@@ -131,7 +144,7 @@ gtk_cell_renderer_class_init (GtkCellRendererClass *class)
 				   PROP_YALIGN,
 				   g_param_spec_float ("yalign",
 						       _("yalign"),
-						       _("The y-align."),
+						       _("The y-align"),
 						       0.0,
 						       1.0,
 						       0.5,
@@ -142,7 +155,7 @@ gtk_cell_renderer_class_init (GtkCellRendererClass *class)
 				   PROP_XPAD,
 				   g_param_spec_uint ("xpad",
 						      _("xpad"),
-						      _("The xpad."),
+						      _("The xpad"),
 						      0,
 						      100,
 						      2,
@@ -153,7 +166,7 @@ gtk_cell_renderer_class_init (GtkCellRendererClass *class)
 				   PROP_YPAD,
 				   g_param_spec_uint ("ypad",
 						      _("ypad"),
-						      _("The ypad."),
+						      _("The ypad"),
 						      0,
 						      100,
 						      2,
@@ -164,7 +177,7 @@ gtk_cell_renderer_class_init (GtkCellRendererClass *class)
 				   PROP_WIDTH,
 				   g_param_spec_int ("width",
 						     _("width"),
-						     _("The fixed width."),
+						     _("The fixed width"),
 						     -1,
 						     100,
 						     -1,
@@ -175,7 +188,7 @@ gtk_cell_renderer_class_init (GtkCellRendererClass *class)
 				   PROP_HEIGHT,
 				   g_param_spec_int ("height",
 						     _("height"),
-						     _("The fixed height."),
+						     _("The fixed height"),
 						     -1,
 						     100,
 						     -1,
@@ -186,7 +199,7 @@ gtk_cell_renderer_class_init (GtkCellRendererClass *class)
 				   PROP_IS_EXPANDER,
 				   g_param_spec_boolean ("is_expander",
 							 _("Is Expander"),
-							 _("Row has children."),
+							 _("Row has children"),
 							 FALSE,
 							 G_PARAM_READABLE |
 							 G_PARAM_WRITABLE));
@@ -200,6 +213,29 @@ gtk_cell_renderer_class_init (GtkCellRendererClass *class)
 							 FALSE,
 							 G_PARAM_READABLE |
 							 G_PARAM_WRITABLE));
+
+  g_object_class_install_property (object_class,
+				   PROP_CELL_BACKGROUND,
+				   g_param_spec_string ("cell_background",
+							_("Cell background color name"),
+							_("Cell background color as a string"),
+							NULL,
+							G_PARAM_WRITABLE));
+
+  g_object_class_install_property (object_class,
+				   PROP_CELL_BACKGROUND_GDK,
+				   g_param_spec_boxed ("cell_background_gdk",
+						       _("Cell background color"),
+						       _("Cell background color as a GdkColor"),
+						       GDK_TYPE_COLOR,
+						       G_PARAM_READABLE | G_PARAM_WRITABLE));
+
+
+#define ADD_SET_PROP(propname, propval, nick, blurb) g_object_class_install_property (object_class, propval, g_param_spec_boolean (propname, nick, blurb, FALSE, G_PARAM_READABLE | G_PARAM_WRITABLE))
+
+  ADD_SET_PROP ("cell_background_set", PROP_CELL_BACKGROUND_SET,
+                _("Cell background set"),
+                _("Whether this tag affects the cell background color"));
 }
 
 static void
@@ -209,6 +245,10 @@ gtk_cell_renderer_get_property (GObject     *object,
 				GParamSpec  *pspec)
 {
   GtkCellRenderer *cell = GTK_CELL_RENDERER (object);
+  GtkCellRendererInfo *cellinfo;
+
+  cellinfo = g_object_get_data (object,
+		                GTK_CELL_RENDERER_INFO_KEY);
 
   switch (param_id)
     {
@@ -237,11 +277,26 @@ gtk_cell_renderer_get_property (GObject     *object,
       g_value_set_int (value, cell->height);
       break;
     case PROP_IS_EXPANDER:
-      g_value_set_int (value, cell->is_expander);
+      g_value_set_boolean (value, cell->is_expander);
       break;
     case PROP_IS_EXPANDED:
-      g_value_set_int (value, cell->is_expanded);
+      g_value_set_boolean (value, cell->is_expanded);
       break;
+    case PROP_CELL_BACKGROUND_GDK:
+      {
+	GdkColor color;
+
+	color.red = cellinfo->cell_background.red;
+	color.green = cellinfo->cell_background.green;
+	color.blue = cellinfo->cell_background.blue;
+
+	g_value_set_boxed (value, &color);
+      }
+      break;
+    case PROP_CELL_BACKGROUND_SET:
+      g_value_set_boolean (value, cell->cell_background_set);
+      break;
+    case PROP_CELL_BACKGROUND:
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
       break;
@@ -289,12 +344,63 @@ gtk_cell_renderer_set_property (GObject      *object,
     case PROP_IS_EXPANDED:
       cell->is_expanded = g_value_get_boolean (value);
       break;
+    case PROP_CELL_BACKGROUND:
+      {
+	GdkColor color;
+
+	if (!g_value_get_string (value))
+	  set_cell_bg_color (cell, NULL);
+	else if (gdk_color_parse (g_value_get_string (value), &color))
+	  set_cell_bg_color (cell, &color);
+	else
+	  g_warning ("Don't know color `%s'", g_value_get_string (value));
+
+	g_object_notify (object, "cell_background_gdk");
+      }
+      break;
+    case PROP_CELL_BACKGROUND_GDK:
+      set_cell_bg_color (cell, g_value_get_boxed (value));
+      break;
+    case PROP_CELL_BACKGROUND_SET:
+      cell->cell_background_set = g_value_get_boolean (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
       break;
     }
 }
 
+static void
+set_cell_bg_color (GtkCellRenderer *cell,
+		   GdkColor        *color)
+{
+  GtkCellRendererInfo *cellinfo;
+
+  cellinfo = g_object_get_data (G_OBJECT (cell),
+		                GTK_CELL_RENDERER_INFO_KEY);
+
+  if (color)
+    {
+      if (!cell->cell_background_set)
+        {
+	  cell->cell_background_set = TRUE;
+	  g_object_notify (G_OBJECT (cell), "cell_background_set");
+	}
+
+      cellinfo->cell_background.red = color->red;
+      cellinfo->cell_background.green = color->green;
+      cellinfo->cell_background.blue = color->blue;
+    }
+  else
+    {
+      if (cell->cell_background_set)
+        {
+	  cell->cell_background_set = FALSE;
+	  g_object_notify (G_OBJECT (cell), "cell_background_set");
+	}
+    }
+}
+
 /**
  * gtk_cell_renderer_get_size:
  * @cell: a #GtkCellRenderer
@@ -338,7 +444,13 @@ gtk_cell_renderer_get_size (GtkCellRenderer *cell,
       *height = cell->height;
     }
 
-  GTK_CELL_RENDERER_GET_CLASS (cell)->get_size (cell, widget, cell_area, x_offset, y_offset, real_width, real_height);
+  GTK_CELL_RENDERER_GET_CLASS (cell)->get_size (cell,
+						widget,
+						cell_area,
+						x_offset,
+						y_offset,
+						real_width,
+						real_height);
 }
 
 /**
@@ -369,9 +481,34 @@ gtk_cell_renderer_render (GtkCellRenderer     *cell,
 			  GdkRectangle        *expose_area,
 			  GtkCellRendererState flags)
 {
+  gboolean selected = FALSE;
+  GtkCellRendererInfo *cellinfo;
+
+  cellinfo = g_object_get_data (G_OBJECT (cell),
+		                GTK_CELL_RENDERER_INFO_KEY);
+
   g_return_if_fail (GTK_IS_CELL_RENDERER (cell));
   g_return_if_fail (GTK_CELL_RENDERER_GET_CLASS (cell)->render != NULL);
 
+  selected = (flags & GTK_CELL_RENDERER_SELECTED) == GTK_CELL_RENDERER_SELECTED;
+
+  if (cell->cell_background_set && !selected)
+    {
+      GdkColor color;
+      GdkGC *gc;
+
+      color.red = cellinfo->cell_background.red;
+      color.green = cellinfo->cell_background.green;
+      color.blue = cellinfo->cell_background.blue;
+
+      gc = gdk_gc_new (window);
+      gdk_gc_set_rgb_fg_color (gc, &color);
+      gdk_draw_rectangle (window, gc, TRUE,
+                          background_area->x, background_area->y,
+                          background_area->width, background_area->height);
+      g_object_unref (gc);
+    }
+
   GTK_CELL_RENDERER_GET_CLASS (cell)->render (cell,
 					      window,
 					      widget,
